#!/bin/sh

DISPLAY=127.0.0.1
COMMUNITY=public
DFYELLOW=90
DFRED=95
CPUYELLOW=90
CPURED=95
MEMYELLOW=90
MEMRED=95

export DISPLAY COMMUNITY DFYELLOW DFRED CPUYELLOW CPURED MEMYELLOW MEMRED

if test -z "$BB"; then
	echo "BB not set; test run"
	BB=echo
	BBHOME=/usr/lib/hobbit/server
	export BB BBHOME
fi

# Update the column status
# status hostname testname color message
status()
{
	hostname=`echo "$1"|tr . ,`
	$BB $DISPLAY "status $hostname.$2 $3 `date`
$4
"
}

# Send performance data separately for better control
# trends hostname rrdfile data
trends()
{
	hostname=`echo "$1"|tr . ,`
	$BB $DISPLAY "data $hostname.trends
[$2.rrd]
$3
"
}

# get_snmp host oid
get_snmp()
{
	snmpwalk -v 1 -c "$COMMUNITY" -On $1 $2|sed -e 's,^[^:]*: ,,'|tr -d \"
}

# The result from snmpwalk is a very long text where each line looks like:
# .1.3.6.more.numbers = Type: value
get_value()
{
	echo "$snmp_results"|grep -e "^$1."|sed -e 's,^[^:]*: ,,'|tr -d \"
}

# Floating point arithmetic using awk
# aexpr "1.2+1.3"
# prints 2.5
aexpr()
{
#	echo "aexpr '$1'" 1>&2
	awk "END { print $1 }" < /dev/null
}

# acmp "1.2 < 1.3"
# returns 0 if true, 1 if false
acmp()
{
#	echo "acmp '$1'" 1>&2
	awk "END { exit ! ($1) }" < /dev/null
}

enterprises_oid=.1.3.6.1.4.1
storage_oid=$enterprises_oid.24681
storageSystem_oid=$storage_oid.1
SystemInfo_oid=$storageSystem_oid.2
SystemCPU_Usage_oid=$SystemInfo_oid.1		# System CPU usage
SystemTotalMem_oid=$SystemInfo_oid.2		# System total memory
SystemFreeMem_oid=$SystemInfo_oid.3		# System free memory
SystemUptime_oid=$SystemInfo_oid.4		# Time since host was initialized
CPU_Temperature_oid=$SystemInfo_oid.5		# CPU temperature
SystemTemperature_oid=$SystemInfo_oid.6		# System temperature
IfNumber_oid=$SystemInfo_oid.8			# Number of network interfaces
SystemIfTable_oid=$SystemInfo_oid.9		# List of interface entries
IfEntryDef_oid=$SystemIfTable_oid.1		# An interface entry
IfIndex_oid=$IfEntry_oid.1			# A unique value for each interface
IfDescr_oid=$IfEntry_oid.2			# Textual string containing info about the interface
IfPacketsReceived_oid=$IfEntry_oid.3		# System packets received
IfPacketsSent_oid=$IfEntry_oid.4		# System packets sent
IfErrorPackets_oid=$IfEntry_oid.5		# System error packets
HdNumber_oid=$SystemInfo_oid.10			# Number of hard disks
SystemHdTable_oid=$SystemInfo_oid.11		# List of hard disk entries
HdEntry_oid=$SystemHdTable_oid.1		# A hard disk entry
HdIndex_oid=$HdEntry_oid.1			# A unique value for each disk
HdDescr_oid=$HdEntry_oid.2			# Textual string containing info about the disk
HdTemperature_oid=$HdEntry_oid.3		# Hard disk temperature
HdStatus_oid=$HdEntry_oid.4			# 0=ready, -5=noDisk, -6=invalid, -9=rwError, -4=unknown
HdModel_oid=$HdEntry_oid.5			# Hard disk model
HdCapacity_oid=$HdEntry_oid.6			# Hard disk capacity
HdSmartInfo_oid=$HdEntry_oid.7			# SMART information
ModelName_oid=$SystemInfo_oid.12		# Model name
HostName_oid=$SystemInfo_oid.13			# Host name
SysFanNumber_oid=$SystemInfo_oid.14		# Number of system fans
SystemFanTable_oid=$SystemInfo_oid.15		# A list of system fans
SysFanEntry_oid=$SystemFanTable_oid.1		# A system fan entry
SysFanIndex_oid=$SysFanEntry_oid.1		# A unique value for each system fan
SysFanDescr_oid=$SysFanEntry_oid.2		# Textual string containing info about the fan
SysFanSpeed_oid=$SysFanEntry_oid.3		# System fan speed
SysVolumeNumber_oid=$SystemInfo_oid.16		# Number of system volumes
SystemVolumeTable_oid=$SystemInfo_oid.17	# List of volume entries
SysVolumeEntry_oid=$SystemVolumeTable_oid.1	# A system volume entry
SysVolumeIndex_oid=$SysVolumeEntry_oid.1	# A unique value for each system volume
SysVolumeDescr_oid=$SysVolumeEntry_oid.2	# Textual string containing info about the volume
SysVolumeFS_oid=$SysVolumeEntry_oid.3		# System volume file system
SysVolumeTotalSize_oid=$SysVolumeEntry_oid.4	# System volume total size
SysVolumeFreeSize_oid=$SysVolumeEntry_oid.5	# System volume free size
SysVolumeStatus_oid=$SysVolumeEntry_oid.6	# System volume status


# Store temperatures in disks.temp$i.rrd where $i is the number of the disk
# disks hostname
disks()
{
	color=green
	tempdata=
	i=1
	number=`get_value "$HdIndex_oid.$i"`
	if test -z "$number"; then
		status $1 disks clear "No disk info"
		return
	fi
	message="<table>
<tr><th>&nbsp;</th><th>#</th><th>Description</th><th>Temperature</th><th>Status</th><th>Model</th><th>Capacity</th><th>SMART info</th></tr>"
	while test ! -z "$number"; do
		bullet=green
		descr=`get_value "$HdDescr_oid.$i"`
		# Temperature is returned as "35 C/95 F" - strip everything after initial space
		temp=`get_value "$HdTemperature_oid.$i"|awk '{print $1}'`
		status=`get_value "$HdStatus_oid.$i"`
		case "$status" in
		0 )
			status_text="Ready"
			;;
		-5 )
			status_text="No disk"
			;;
		-6 )
			status_text="Invalid"
			;;
		-9 )
			status_text="RW error"
			;;
		-4 )
			status_text="Unknown"
			;;
		* )
			status_text="Unknown status $status"
			;;
		esac
		model=`get_value "$HdModel_oid.$i"`
		capacity=`get_value "$HdCapacity_oid.$i"`
		smart=`get_value "$HdSmartInfo_oid.$i"`
		if test "$smart" != "GOOD"; then
			bullet=yellow
			if test "$color" != red; then
				color=yellow
			fi
		fi
		if test "$status" != "0"; then
			bullet=red
			color=red
		fi
		trends $1 "disks.temp$i" "DS:temp:GAUGE:600:U:U $temp"
		message="$message
<tr><td>&$bullet</td><td>$number</td><td>$descr</td><td>$temp</td><td>$status_text</td><td>$model</td><td>$capacity</td><td>$smart</td></tr>"
		i=`expr $i + 1`
		number=`get_value "$HdIndex_oid.$i"`
	done
	message="$message
</table>"
	status $1 disks $color "$message"
}

# Store rpm in fans.rpm$i.rrd where $i is the number of the fan
# fans hostname
fans()
{
	color=green
	i=1
	number=`get_value "$SysFanIndex_oid.$i"`
	if test -z "$number"; then
		status $1 fans clear "No fan info"
		return
	fi
	message="<table>
<tr><th>&nbsp;</th><th>#</th><th>Description</th><th>Speed</th></tr>"
	while test ! -z "$number"; do
		bullet=green
		descr=`get_value "$SysFanDescr_oid.$i"`
		# Speed is returned as "1163 RPM" - only keep first field
		speed=`get_value "$SysFanSpeed_oid.$i"|awk '{print $1}'`
		trends $1 "fans.rpm$i" "DS:rpm:GAUGE:600:U:U $speed"
		message="$message
<tr><td>&$bullet</td><td>$number</td><td>$descr</td><td>$speed</td></tr>"
		i=`expr $i + 1`
		number=`get_value "$SysFanIndex_oid.$i"`
	done
	message="$message
</table>"
	status $1 fans $color "$message"
}

# Store temperature in temperature.temp$i.rrd where $i is the number of the temperature
# temperature ipaddr hostname
temperature()
{
	color=green
	# For whatever reason, these two don't show up in snmp_results
	# Temperature is returned as "35 C/95 F" - strip everything after initial space
	CPU_Temperature=`get_snmp $1 "$CPU_Temperature_oid"|awk '{print $1}'`
	SystemTemperature=`get_snmp $1 "$SystemTemperature_oid"|awk '{print $1}'`

	message="<table>
<tr><td>CPU Temperature</td><td>$CPU_Temperature</td></tr>
<tr><td>System Temperature</td><td>$SystemTemperature</td></tr>
</table>"
	if test ! -z "$CPU_Temperature"; then
		trends $2 "temperature.CPU" "DS:temp:GAUGE:600:U:U $CPU_Temperature"
	fi
	if test ! -z "$SystemTemperature"; then
		trends $2 "temperature.System" "DS:temp:GAUGE:600:U:U $SystemTemperature"
	fi

	status $2 temperature $color "$message"
}

convert_size()
{
	read value unit
#echo "value='$value'"
#echo "unit='$unit'"
	case "$unit" in
	TB )
		aexpr "$value*1000000000000"
		;;
	GB )
		aexpr "$value*1000000000"
		;;
	MB )
		aexpr "$value*1000000"
		;;
	kB )
		aexpr "$value*1000"
		;;
	* )
		$value
		;;
	esac
}

# Store percent full in volumes.space$i.rrd where $i is the number of the volume
# volumes hostname
volumes()
{
	color=green
	i=1
	number=`get_value "$SysVolumeIndex_oid.$i"`
	if test -z "$number"; then
		status $1 volumes clear "No volume info"
		return
	fi

	message="<table>
<tr><th>&nbsp;</th><th>#</th><th>Description</th><th>File System</th><th>Total Size</th><th>Free Size</th><th>Used %</th><th>Status</th></tr>"
	while test ! -z "$number"; do
		bullet=green
		descr=`get_value "$SysVolumeDescr_oid.$i"`
		fs=`get_value "$SysVolumeFS_oid.$i"`
		# Size and free are returned as "5.37 TB", use awk to convert
		size=`get_value "$SysVolumeTotalSize_oid.$i"|convert_size`
		free=`get_value "$SysVolumeFreeSize_oid.$i"|convert_size`
		status=`get_value "$SysVolumeStatus_oid.$i"`
		space=`aexpr "100 - 100 * $free / $size"`
		trends $1 "volumes.space$i" "DS:space:GAUGE:600:U:U $space"
		if acmp "$space > $DFYELLOW"; then
			bullet=yellow
			color=yellow
		fi
		if acmp "$space > $DFRED"; then
			bullet=red
			color=red
		fi
		if test "$status" != "Ready"; then
			bullet=red
			color=red
		fi
		message="$message
<tr><td>&$bullet</td><td>$number</td><td>$descr</td><td>$fs</td><td>$size</td><td>$free</td><td>$space</td><td>$status</td></tr>"
		i=`expr $i + 1`
		number=`get_value "$SysVolumeIndex_oid.$i"`
	done
	message="$message
</table>"
	status $1 volumes "$color" "$message"
}

# qnap ipaddr hostname
qnap()
{
	snmp_results=`snmpwalk -v 1 -c "$COMMUNITY" -On $1 $SystemInfo_oid`
	if test -z "$snmp_results"; then
		status $2 status red
		exit
	fi
	# For whatever reason, these don't show up in snmp_results
	CPU_Usage=`get_snmp $1 $SystemCPU_Usage_oid|awk '{print $1}'`
	if acmp "$CPU_Usage >= $CPURED"; then
		color=red
	elif acmp "$CPU_Usage >= $CPUYELLOW"; then
		color=yellow
	else
		color=green
	fi
	status $2 cpu $color "CPU Usage: $CPU_Usage %"
	trends $2 la "DS:la:GAUGE:600:U:U $CPU_Usage"
	TotalMem=`get_snmp $1 $SystemTotalMem_oid|convert_size`
	FreeMem=`get_snmp $1 $SystemFreeMem_oid|convert_size`
	MemPct=`aexpr "100-100*$FreeMem/$TotalMem"`
	if acmp "$MemPct >= $MEMRED"; then
		color=red
	elif acmp "$MemPct >= $MEMYELLOW"; then
		color=yellow
	else
		color=green
	fi
	status $2 memory $color "
Total Memory: $TotalMem
Free Memory:  $FreeMem
Percent Used: $MemPct %
"
	trends $2 memory.real "DS:realmempct:GAUGE:600:U:U $MemPct"
	Uptime=`get_snmp $1 $SystemUptime_oid`
	ModelName=`get_snmp $1 $ModelName_oid`
	HostName=`get_snmp $1 $HostName_oid`
	status $2 status green "
Model:    $ModelName
Hostname: $HostName
Uptime:   $Uptime
"

	disks $2
	fans $2
	temperature $1 $2
	volumes $2
}

$BBHOME/bin/bbhostgrep qnap |
while read HOSTIP HOSTNAME JUNK; do
	qnap $HOSTIP $HOSTNAME
done
